#!C:\Users\Thomas\AppData\Local\Programs\Python\Python35-32\python
########################################
# program reads the photo transistor   #
# file: PhotoTransistorClass.pyw       #
# Author: Th. Kaffka, Cologne, Germany #
# Date: 15/03/2021                     #
########################################
import sys
sys.path.append("/home/pi/Ae/Python/Robots/Interface")
import tkinter as tk
from threading import Thread
import RPi.GPIO as GPIO
import time
from PhotoTransistor import PhotoTransistor

# define constants.
PROG_NAME = "Photo Transistor Class"
PROG_VERSION = "0.1"
PIC_WIDTH = 250
PIC_HEIGHT = 50

SPICLK = 3
SPIMISO = 5
SPIMOSI = 16
SPICS = 12

ptrans = PhotoTransistor(0, SPICLK, SPIMOSI, SPIMISO, SPICS)

# define globals.
processing_go = False

class PhotoTransistorClass(tk.Frame):

   def __init__(self, master=None):
      tk.Frame.__init__(self, master)
      self.grid()
      self.createWidgets()
      
   def createWidgets(self):
      """ create the widgets of the surface. """
      self.picture_ground = tk.Canvas(self, height=str(PIC_HEIGHT), width=str(PIC_WIDTH), background="#ADD8E6")
      self.picture_ground.create_text(PIC_WIDTH - 140, PIC_HEIGHT-20, text=PROG_NAME, fill="#00008B", tag="coord")
      self.picture_ground.pack()
      self.startButton = tk.Button(self, text="Start", command=self.startRobot).pack(fill=tk.X)
      self.stopButton = tk.Button(self, text="Stop", command=self.stopRobot).pack(fill=tk.X)
      self.quitButton = tk.Button(self, text="Quit", command=self.winClose).pack(fill=tk.X)
      self.messageLabel = tk.Label(self, text="")
      self.messageLabel.pack(fill=tk.X)

   def winClose(self):
      """ exits the program. """
      if (processing_go): return
      # delete the objects begin.
      global ptrans
      del ptrans
      # delete the objects end.
      GPIO.cleanup()
      self.quit()
      self.destroy()

   def startRobot(self):
      """ starts the robots processing. """
      global processing_go
      self.messageLabel.configure(text = "Start Robot processing.")
      processing_go = True
      self.t = Thread(target=self.loopRobotProcessing, args=())
      self.t.start()

   def stopRobot(self):
      """ stops the robots processing. """
      global processing_go
      if (not processing_go): return
      self.messageLabel.configure(text = "Stop Robot processing.")
      processing_go = False

   def loopRobotProcessing(self):
      """ processes the robot in a loop. """
      while (processing_go):
         # enter robots controlling statements begin.
         value = ptrans.readValue()
         print(value)
         time.sleep(0.5)
         # enter robots controlling statements end.
         
app = PhotoTransistorClass()
app.master.title(PROG_NAME + " (" + PROG_VERSION + ")")
app.mainloop()
