#!/usr/bin/python3
####################################################
# Program morses.                                  #
# file: MorseCode.py                               #
# Author: Th. Kaffka, Cologne, Germany             #
# Date: 09/01/2021                                 #
####################################################
import sys
sys.path.append("/home/pi/Ae/Python/Robots/Interface")
import RPi.GPIO as GPIO
from threading import Thread
import time
from Led import Led
from TouchSensor import TouchSensor

led = Led(13)
speaker = 0
switch = TouchSensor(12)

TONE_A4 = 440

ok = True
play_tone = False
tone_init = False

morse_alphabet = [".-", "-...", "-.-.", "-..", ".", "..-.", "--.", "....", \
                  "..", ".---", "-.-", ".-..", "--", "-.", "---", ".--.", \
                  "--.-", ".-.", "...", "-", "..-", "...-", ".--", "-..-", \
                  "-.--", "--.."]

char_alphabet = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", \
                 "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z"]

###############################
# initialize the piezo speaker.
def initSpeaker(pin_number):
    global speaker
    GPIO.setmode(GPIO.BOARD)
    GPIO.setup(pin_number, GPIO.OUT)
    speaker = GPIO.PWM(pin_number, 100)
    speaker.start(0)

###############
# make a sound.
def sound():
    global ok, tone_init, play_tone
    while(ok):
        if (tone_init):
            speaker.ChangeDutyCycle(50)
            speaker.ChangeFrequency(TONE_A4)
            tone_init = False
            play_tone = True
        if (play_tone):
            pass
        else:
            speaker.ChangeDutyCycle(0)
            

print("Start")

try:
    initSpeaker(15)
    t = Thread(target=sound, args=())
    t.setDaemon(True)
    t.start()
    
    press = False
    release = False
    duration = 0
    pause = 0
    get_pause = False
    character = ""
    
    while (True):
        if (switch.isTouch()):
            led.ledOn()
            if (not play_tone):
                tone_init = True
            if (not press):
                duration = time.time()
                press = True
        elif (press):
            led.ledOff()
            play_tone = False
            duration = time.time() - duration
            release = True
        
        if (release and not get_pause):
            pause = time.time()
            get_pause = True
        elif (press and get_pause):
            get_pause = False
            pause = time.time() - pause
            if (pause > 1.0): # end char.
                try:
                    ndx = morse_alphabet.index(character)
                except ValueError:
                    ndx = -1
                except KeyboardInterrupt:
                    break
                if (ndx > -1):
                    print (char_alphabet[ndx], end="")
                else:
                    print ("~", end="")
                character = ""
            if (pause > 2.0): # end word.
                print()

        if (press and release):
            press = False
            release = False
            if (duration < 0.20):
                character += "."
            else:
                character += "-"
                
except KeyboardInterrupt:
    pass
finally:
    ok = False
    GPIO.cleanup()

print()
print("End")
