#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Python/Robots/Interface")
import RPi.GPIO as GPIO
import time
from GearMotor import GearMotor
from Vehicle import Vehicle
from Led import Led
from TouchSensor import TouchSensor
from PiezoSpeaker import PiezoSpeaker

####################################################
# Program to move the platonian areas.             #
# file: Platon.py                                  #
# Author: Th. Kaffka, Cologne, Germany             #
# Date: 04/04/2016                                 #
####################################################

TRIANGLE = 3
QUARDER = 4
PENTAGON = 5
HEXAGON = 6

PMWA = 40 
AIN2 = 38
AIN1 = 36
ENCA = 32

PMWB = 37 
BIN1 = 35
BIN2 = 33
ENCB = 31

button = TouchSensor(11)
led = Led(13) 
speaker = PiezoSpeaker(15)

motor_a = GearMotor(PMWA, AIN2, AIN1, ENCA)
motor_b = GearMotor(PMWB, BIN1, BIN2, ENCB)
vehicle = Vehicle(3.2, 19.5, motor_a, motor_b)

start = True
led_on = False

figure = 0
count = 0
angle = 0
distance = 0

speaker.beepMessageReady()

try:

   while(True):
      if (start and button.isTouch()):
         start = False
         led.ledOff()
         speaker.beepMessageOk()
         time.sleep(3)
         vehicle.setSpeed(25)
         continue
      elif(start):
         led_on = not led_on
         if (led_on):
            led.ledOn()
         else:
            led.ledOff()
         time.sleep(0.5)
         continue

      if (count == 0):
         speaker.beepMessageReady()

         if (figure == 0):
            figure = TRIANGLE
         elif (figure == TRIANGLE):
            figure = QUARDER
         elif (figure == QUARDER):
            figure = PENTAGON
         elif (figure == PENTAGON):
            figure = HEXAGON
         else:
            start = True
            figure = 0
            continue

         count = figure
         angle = 360 / figure
         distance = 1800 / figure

      vehicle.forward(distance)
      time.sleep(0.3)
      vehicle.rotate(angle)
      time.sleep(0.3)
      count -= 1

except KeyboardInterrupt:
   pass
finally:
   GPIO.cleanup()


