#!/usr/bin/python3
####################################################
# Program to find the brightest light source.      #
# file: LightSearch.py                             #
# Author: Th. Kaffka, Cologne, Germany             #
# Date: 15/03/2021                                 #
####################################################
import sys
sys.path.append("/home/pi/Ae/Python/Robots/Interface")
import RPi.GPIO as GPIO
import time
from GearMotor import GearMotor
from Vehicle import Vehicle
from Led import Led
from TouchSensor import TouchSensor
from PiezoSpeaker import PiezoSpeaker
from PhotoTransistor import PhotoTransistor

PMWA = 40 
AIN2 = 38
AIN1 = 36
ENCA = 32

PMWB = 37 
BIN1 = 35
BIN2 = 33
ENCB = 31

SPICLK = 3
SPIMISO = 5
SPIMOSI = 16
SPICS = 12

button = TouchSensor(11)
led = Led(13) 
speaker = PiezoSpeaker(15)

motor_a = GearMotor(PMWA, AIN2, AIN1, ENCA)
motor_b = GearMotor(PMWB, BIN1, BIN2, ENCB)
vehicle = Vehicle(3.2, 19.5, motor_a, motor_b)
ptrans = PhotoTransistor(0, SPICLK, SPIMOSI, SPIMISO, SPICS)

start = True
led_on = False

speaker.beepMessageReady()

try:

   while(True):
      if (start and button.isTouch()):
         start = False
         led.ledOff()
         speaker.beepMessageOk()
         time.sleep(3)
         vehicle.setSpeed(30)
         continue
      elif(start):
         led_on = not led_on
         if (led_on):
            led.ledOn()
         else:
            led.ledOff()
         time.sleep(0.5)
         continue
      elif(not start and button.isTouch()):
         start = True
         speaker.beepMessageEnd()
         continue

      # light search
      max_light = 0
      max_angle = 0;
      for ndx in range(12):
         vehicle.rotate(30)
         time.sleep(0.3);
         value = ptrans.readValue()
         if (value > max_light):
            max_light = value
            max_angle = (ndx + 1) * 30;

      # go to max. light value.
      vehicle.rotate(max_angle)
      time.sleep(0.3);
      vehicle.forward(200)
      speaker.beepMessageReady()

except KeyboardInterrupt:
   pass
finally:
   GPIO.cleanup()
