#!/usr/bin/python
########################################
# class rasprberry pi camera module    #
# file: Camera.py                      #
# Author: Th. Kaffka, Cologne, Germany #
# Date: 21.01.2021                     #
########################################
import io
import time
import picamera
from PIL import Image

class Camera:

   def __init__(self):
      self.cam = picamera.PiCamera()
      self.cam.hflip = True
      self.cam.vflip = True
      self.cam.image_effect = "posterise"
      self.setResolution(1024, 768)
      time.sleep(2)

   def __del__(self):
      self.cam.close()
      
   def setResolution(self, w, h):
      self.cam.resolution = (w, h)

   def capture(self, w, h):
      stream = io.BytesIO()
      self.cam.capture(stream, format="bmp", resize=(w, h))
      stream.seek(0)
      return stream

   def captureList(self, w, h):
      stream = self.capture(w, h)
      image = Image.open(stream)
      return list(image.getdata())
