#!/usr/bin/python
########################################
# class lego vehicle                   #
# file: LegoVehicle.py                 #
# Author: Th. Kaffka, Cologne, Germany #
# Date: 21.10.2020                     #
########################################
import brickpi3 
import math

BP = brickpi3.BrickPi3()

WHEEL_SIZE_NXT = 5.6
WHEEL_SIZE_EV3 = 4.32
BIG_WHEEL_SIZE_EV3 = 8.5
CHAIN_SIZE_EV3 = 3.6

class LegoVehicle(object):

   def __init__(self, wheel_diameter, track_size, motor_left, motor_right):
      self.wheel_diameter = wheel_diameter
      self.track_size = track_size
      self.motor_left = motor_left
      self.motor_right = motor_right
      self.power = 0
      self.turn_ratio = track_size / wheel_diameter * 2
      self.degree_per_distance = 360 / (math.pi * wheel_diameter)
          

   def __del__(self):
      pass
      
   def getSpeed(self):
      return self.power

   def setSpeed(self, power):
      self.power = abs(power)
      self.motor_left.setSpeed(self.power)
      self.motor_right.setSpeed(self.power)

   def getLeftEncoder(self):
      return self.motor_left.getEncoder()

   def getRightEncoder(self):
      return self.motor_right.getEncoder()

   def getLeftTachoCount(self):
      return self.motor_left.getTachoCount()

   def getRightTachoCount(self):
      return self.motor_right.getTachoCount()

   def resetTachoCount(self):
      self.motor_left.resetTachoCount()
      self.motor_right.resetTachoCount()

   def countTacho(self):
      self.motor_left.countTacho()
      self.motor_right.countTacho()

   def forward(self, fwd=0):
      '''move forward, fwd=distance in cm'''
      if (not fwd == 0):
         fwd = abs(fwd) * self.degree_per_distance
      self.motor_left.forward(fwd)
      self.motor_right.forward(fwd)

   def backward(self, bwd=0):
      '''move back, bwd=distance in cm'''
      if (not bwd == 0):
         bwd = abs(bwd) * self.degree_per_distance
      self.motor_left.backward(bwd)
      self.motor_right.backward(bwd)

   def stop(self):
      '''stop the motor'''
      self.motor_left.stop()
      self.motor_right.stop()

   def isMoving(self):
      return self.motor_left.isMoving() or self.motor_right.isMoving()

   def rotateLeft(self):
      self.motor_left.forward()
      self.motor_right.backward()

   def rotateRight(self):
      self.motor_left.backward()
      self.motor_right.forward()

   def rotate(self, angle):
      '''rotate a given angle (+, -)'''
      rotate_angle = angle * self.turn_ratio
      self.motor_left.rotate(-rotate_angle)
      self.motor_right.rotate(rotate_angle)
