#!/usr/bin/python3
from random import *
from math import exp

########################################
# class neural network                 #
# file: NeuralNetwork.py               #
# Author: Th. Kaffka, Cologne, Germany #
# Date: 06/09/2016                     #
########################################

class NeuralNetwork:

   #########################
   # initializes the object.
   # x_columns: number of the x values.
   # y_columns: number of the y values.
   # epsilon:   value of epsilon.
   # nh:        number of the elements in the hidden layer.
   # examples:  the example array (list).
   # exam_len:  the number of examples.
   # factor_x:  the factor to recalc the x example values.
   # factor_y:  the factor to recalc the y example values.
   def __init__(self, x_columns, y_columns, epsilon, nh, examples, exam_len,\
                factor_x, factor_y):
      self.x_columns = x_columns
      self.y_columns = y_columns
      self.x = []
      self.y = []
      self.d = []
      self.epsilon = epsilon
      self.train_step = 0
      self.nh = nh
      self.h = []
      self.w1 = []
      self.w2 = []
      self.examples = examples
      self.exam_len = exam_len
      self.factor_x = factor_x
      self.factor_y = factor_y
      self.max_error_net = 0.0
      self.min_error_net = 0.0
      self.max_error = 0.0
      self.min_error = 0.0
      self.av_error = 0.0
      self.initNN()

   def __del__(self):
      pass 
      
   #################################
   # initializes the neural network.
   def initNN(self):
      self.x = self.makeArray(self.x_columns)
      self.x.append(1.0) # adds the bias.
      self.y = self.makeArray(self.y_columns)
      self.d = self.makeArray(self.y_columns)
      self.train_step = 0
      self.h = self.makeArray(self.nh)
      self.h.append(1.0) # adds the bias.
      self.w1 = self.makeArray2Dim(self.x_columns + 1, self.nh + 1)
      self.w2 = self.makeArray2Dim(self.nh + 1, self.y_columns)

      # load w1 and w2 with random numbers.
      for i in range(0, self.nh + 1):
         for j in range(0, self.x_columns + 1):
            self.w1[j][i] = uniform(0.0, 1.0)

      for i in range(0, self.y_columns):
         for j in range(0, self.nh + 1):
            self.w2[j][i] = uniform(0.0, 1.0)

   ###################
   # creates an array.
   def makeArray(self, n):
      array = []
      for i in range(0, n):
         array.append(0.0)
      return array

   ##########################
   # creates an 2 dim. array.
   def makeArray2Dim(self, row, column):
      array_main = []
      for i in range(0, row):
         array = []
         for j in range(0, column):
            array.append(0.0)
         array_main.append(array)
      return array_main

   ####################
   # transfer function.
   def t(self, a):
      return 1.0 / (1.0 + exp(-a))

   ############################
   # propagate the errors back.
   def backPropagate(self):
      err_h = self.makeArray(self.nh + 1)
      delta = 0.0
      self.max_error_net = 0.0
      self.min_error_net = 1.0
      for j in range(0, self.y_columns):
         delta = abs(self.d[j] - self.y[j])
         if (delta > self.max_error_net):
            self.max_error_net = delta
         if (delta < self.min_error_net):
            self.min_error_net = delta
         delta = (self.d[j] - self.y[j]) * self.y[j] * (1.0 - self.y[j])
         for k in range(0, self.nh + 1):
            err_h[k] += delta * self.w2[k][j]
            self.w2[k][j] += self.epsilon * delta * self.h[k]
      for k in range(0, self.nh):
         delta = err_h[k] * self.h[k] * (1.0 - self.h[k])
         for i in range(0, self.x_columns + 1):
            self.w1[i][k] += self.epsilon * delta * self.x[i]

   #########################################
   # execute the neural network with values.
   def executeFunction(self, px):
      xlen = len(px)
      for i in range(0, xlen):
         self.x[i] = px[i] * self.factor_x
         if self.x[i] > 1.0:
            self.x[i] = 0.99
      self.execute()
      result = []
      for i in range(0, self.y_columns):
         result.append(self.y[i] / self.factor_y)
      return result

   #############################
   # execute the neural network.
   def execute(self):
      for k in range(0, self.nh):
         a = 0.0
         for i in range(0, self.x_columns + 1):
            a += self.w1[i][k] * self.x[i]
         self.h[k] = self.t(a)
      for j in range(0, self.y_columns):
         a = 0.0
         for k in range(0, self.nh + 1):
            a += self.w2[k][j] * self.h[k]
         self.y[j] = self.t(a)

   #####################
   # learn the examples.
   def learn(self):
      self.max_error = 0.0
      self.min_error = 1.0
      self.av_error = 0.0
      for ex in range(0, self.exam_len):
         for j in range(0, self.x_columns):
            self.x[j] = self.examples[ex][j] * self.factor_x
         for j in range(0, self.y_columns):
            self.d[j] = self.examples[ex][j + self.x_columns] * self.factor_y
         self.execute()
         self.backPropagate()
         if (self.max_error_net > self.max_error):
            self.max_error = self.max_error_net
         if (self.min_error_net < self.min_error):
            self.min_error = self.min_error_net
         self.av_error = self.av_error + self.max_error_net
      self.av_error = self.av_error / self.exam_len
