#!/usr/bin/python
##########################################
# program: recognize cardibnal direction #
# file: RecognizeCardinalDirection.py    #
# Author: Th. Kaffka, Cologne, Germany   #
# Date: 10.11.2020                       #
##########################################
import sys
sys.path.append("/home/pi/Ae/Roboter/Interface")
sys.path.append("/home/pi/Ae/Roboter/lib")
import brickpi3 
import time
from LegoMotor import LegoMotor
from LegoVehicle import *
from Led import Led
from LegoTouchSensor import LegoTouchSensor
from HitechnicCompassSensor import HitechnicCompassSensor
import tool4robot as t4r

BP = brickpi3.BrickPi3()

# define the hardware.
motor_l = LegoMotor(BP.PORT_B)
motor_r = LegoMotor(BP.PORT_C)
ts = LegoTouchSensor(BP.PORT_2)
cs = HitechnicCompassSensor(BP.PORT_3)
vehicle = LegoVehicle(BIG_WHEEL_SIZE_EV3, 12.0, motor_l, motor_r)
time.sleep(10)

###################################
# rotate to the cardinal direction.
def rotateTo(cardinal_d):
    vehicle.setSpeed(9)
    vehicle.rotateLeft()
    direction = cs.getDirection()
    while (not (direction > cardinal_d - 5
                and direction < cardinal_d + 5)):
        direction = cs.getDirection()
        time.sleep(0.05)
    vehicle.stop()
    time.sleep(3)

###################
# move to distance.
def moveTo(cardinal_d):
    vehicle.setSpeed(15)
    vehicle.forward(30)
    while vehicle.isMoving():
        cs.getDirection() # dummy.
        time.sleep(0.1)
    time.sleep(2)
    rotateTo(cardinal_d)

print ("Start")

try:
    t4r.initDirection(cs)
    
    t4r.waitUntilTouch(ts)
    time.sleep(2)

    # prepare the vehicle.
    vehicle.resetTachoCount()

    # rotate to the cardinal directions.
    rotateTo(t4r.compass_left)
    rotateTo(t4r.compass_back)
    rotateTo(t4r.compass_right)
    rotateTo(t4r.compass_straight)

    # drive a quarder.
    moveTo(t4r.compass_left)
    moveTo(t4r.compass_back)
    moveTo(t4r.compass_right)
    moveTo(t4r.compass_straight)

except KeyboardInterrupt:
    pass
finally:
    vehicle.stop()
    BP.reset_all()
    
print ("Ende")
