#!/usr/bin/python
##########################################
# program: wall tracking                 #
# begin driving with ir straight.        #
# file: WallTracking.py                  #
# Author: Th. Kaffka, Cologne, Germany   #
# Date: 17.11.2020                       #
##########################################
import sys
sys.path.append("/home/pi/Ae/Roboter/Interface")
sys.path.append("/home/pi/Ae/Roboter/lib")
import brickpi3 
import time
from LegoMotor import LegoMotor
from LegoVehicle import *
from LegoTouchSensor import LegoTouchSensor
from HitechnicCompassSensor import HitechnicCompassSensor
from LegoInfraRedSensor import LegoInfraRedSensor
import tool4robot as t4r

BP = brickpi3.BrickPi3()

# define the hardware.
motor_l = LegoMotor(BP.PORT_C)
motor_r = LegoMotor(BP.PORT_B)
motor_ir = LegoMotor(BP.PORT_D)
ts = LegoTouchSensor(BP.PORT_2)
cs = HitechnicCompassSensor(BP.PORT_1)
ir = LegoInfraRedSensor(BP.PORT_4)
vehicle = LegoVehicle(BIG_WHEEL_SIZE_EV3, 12.0, motor_l, motor_r)
time.sleep(10)

#define constants.
SPEED = 17
ROT_SPEED = 15
IR_RIGHT = 1
IR_STRAIGHT = 2
WALL_DISTANCE = 25
LEFT = 0
RIGHT = 1
ADJ_INTERVALL = 4

#####################
# turn the IR sensor.
def turnIr(direction):
    if direction == IR_RIGHT:
        motor_ir.setSpeed(25)
        motor_ir.forward()
        enc = motor_ir.getEncoder()
        while(enc < 87):
            enc = motor_ir.getEncoder()
            cs.getDirection() # dummy.
        motor_ir.stop()
    elif direction == IR_STRAIGHT:
        motor_ir.setSpeed(30)
        motor_ir.backward()
        enc = motor_ir.getEncoder()
        while(enc > 3):
            enc = motor_ir.getEncoder()
            cs.getDirection() # dummy.
        motor_ir.stop()

###################################
# rotate to the cardinal direction.
def rotateTo(orientation):
    vehicle.setSpeed(ROT_SPEED)
    if (orientation == LEFT):
        vehicle.rotateRight()
        cardinal_d = t4r.compass_left
    elif (orientation == RIGHT):
        vehicle.rotateLeft()
        cardinal_d = t4r.compass_right
    direction = cs.getDirection()
    while (not (direction > cardinal_d - 5
                and direction < cardinal_d + 5)):
        direction = cs.getDirection()
    vehicle.stop()
    vehicle.setSpeed(SPEED)
    t4r.initDirection(cs)
    time.sleep(2)

print ("Start")

near = True
far = True
adjust = 0

try:
    t4r.waitUntilTouch(ts)
    time.sleep(2)

    # prepare the vehicle.
    t4r.initDirection(cs)
    vehicle.resetTachoCount()
    vehicle.setSpeed(SPEED)
    motor_ir.resetTachoCount()
    turnIr(IR_RIGHT)
    vehicle.forward()
    adjust = time.time() + ADJ_INTERVALL

    while True:
        cs.getDirection() # dummy.
        dist = ir.getDistance()
        if ts.isTouched():
            break
        elif (adjust < time.time()):
            adjust = time.time() + ADJ_INTERVALL
            vehicle.stop()
            turnIr(IR_STRAIGHT)
            dist = ir.getDistance()
            if (dist < WALL_DISTANCE):
                t4r.initDirection(cs)
                rotateTo(LEFT)
            turnIr(IR_RIGHT)
            vehicle.forward()
            far = True
            near = True
        elif (dist > WALL_DISTANCE-2 and dist < WALL_DISTANCE+2):
            pass
        elif (dist < WALL_DISTANCE and near):
            vehicle.stop()
            time.sleep(1)
            vehicle.rotate(3)
            while vehicle.isMoving():
                cs.getDirection() # dummy.
            time.sleep(1)
            vehicle.forward()
            near = False
            far = True
        elif (dist > WALL_DISTANCE and far):
            vehicle.stop()
            time.sleep(1)
            vehicle.rotate(-3)
            while vehicle.isMoving():
                cs.getDirection() # dummy.
            time.sleep(1)
            vehicle.forward()
            far = False
            near = True
            
except KeyboardInterrupt:
    pass
finally:
    vehicle.stop()
    turnIr(IR_STRAIGHT)
    BP.reset_all()
    
print ("Ende")
