#!/usr/bin/python
##########################################
# program: line tracking (classic)       #
# file: LineTrackingClassic.py           #
# Author: Th. Kaffka, Cologne, Germany   #
# Date: 08.12.2020                       #
##########################################
import sys
sys.path.append("/home/pi/Ae/Roboter/Interface")
sys.path.append("/home/pi/Ae/Roboter/lib")
import brickpi3 
import time
from LegoMotor import LegoMotor
from LegoVehicle import *
from LegoTouchSensor import LegoTouchSensor
from LegoColorSensorRaw import LegoColorSensorRaw
import tool4robot as t4r
import tool4file as t4f

BP = brickpi3.BrickPi3()

# define the hardware.
motor_l = LegoMotor(BP.PORT_C)
motor_r = LegoMotor(BP.PORT_B)
ts = LegoTouchSensor(BP.PORT_1)
cls = LegoColorSensorRaw(BP.PORT_3)
vehicle = LegoVehicle(WHEEL_SIZE_EV3, 11.0, motor_l, motor_r)
time.sleep(10)

#define constants.
LIGHT_CORR = 30
STRAIGHT = 0
LEFT = -6
RIGHT = 6
LOG_INTERVALL = 0.3

# define global vars.
line_black = 0
line_white = 0
log_time = 0
track_data = ""

#######################
# initialize the robot.
def setUp():
    global line_black, line_white, line_grey
    print ("set color sensor to black line.")
    t4r.waitUntilTouch(ts)
    time.sleep(2)
    line_black = t4r.sumColor(cls.getColorRaw())
    print (line_black)

    print ("set color sensor to white area.")
    t4r.waitUntilTouch(ts)
    time.sleep(2)
    line_white = t4r.sumColor(cls.getColorRaw())
    print (line_white)

    t4r.waitUntilTouch(ts)
    time.sleep(2)

def logData(light, rotate):
    global track_data
    track_data += str(light) + ", " + str(rotate) + "\n"

print ("Start")

try:
    t4r.waitUntilTouch(ts)
    time.sleep(2)

    setUp()

    vehicle.setSpeed(20)
    vehicle.forward()
    log_time = time.time() + LOG_INTERVALL

    while not ts.isTouched():
        time.sleep(0.02)
        curr_light = t4r.sumColor(cls.getColorRaw())
        if (curr_light < line_black + LIGHT_CORR
                and curr_light > line_black - LIGHT_CORR):
            logData(curr_light, RIGHT)
            vehicle.stop()
            vehicle.rotate(RIGHT)
            while vehicle.isMoving():
                pass
            vehicle.forward()
        elif (curr_light < line_white + LIGHT_CORR
                and curr_light > line_white - LIGHT_CORR):
            logData(curr_light, LEFT)
            vehicle.stop()
            vehicle.rotate(LEFT)
            while vehicle.isMoving():
                pass
            vehicle.forward()
        else:
            if (log_time < time.time()):
                logData(curr_light, STRAIGHT)
                log_time = time.time() + LOG_INTERVALL

except KeyboardInterrupt:
    pass
finally:
    vehicle.stop()
    BP.reset_all()
    t4f.saveContent("../data/", "Data.dat", track_data)
    
print ("End")


