#!/usr/bin/python
##########################################
# program: line tracking (classic)       #
# file: LineTrackingClassic.py           #
# Author: Th. Kaffka, Cologne, Germany   #
# Date: 08.12.2020                       #
##########################################
import sys
sys.path.append("/home/pi/Ae/Roboter/Interface")
sys.path.append("/home/pi/Ae/Roboter/lib")
import brickpi3 
import time
from LegoMotor import LegoMotor
from LegoVehicle import *
from LegoTouchSensor import LegoTouchSensor
from LegoColorSensorRaw import LegoColorSensorRaw
import tool4robot as t4r
from NeuralNetwork import NeuralNetwork

BP = brickpi3.BrickPi3()

# define the hardware.
motor_l = LegoMotor(BP.PORT_C)
motor_r = LegoMotor(BP.PORT_B)
ts = LegoTouchSensor(BP.PORT_1)
cls = LegoColorSensorRaw(BP.PORT_3)
vehicle = LegoVehicle(WHEEL_SIZE_EV3, 11.0, motor_l, motor_r)
time.sleep(10)

#define constants.
STRAIGHT = 0
LEFT = -6
RIGHT = 6

# insert your values begin.
factor = 0.001736111111111111
nn = NeuralNetwork(1, 1, 0.3, 6, [], 0, factor, 1)
nn.w1[0][0] = -0.2869167977166518
nn.w1[1][0] = -2.7228944058343165
nn.w1[0][1] = -90.10890415710287
nn.w1[1][1] = 11.127069654300032
nn.w1[0][2] = -153.63807649454017
nn.w1[1][2] = 122.44646071829585
nn.w1[0][3] = -0.2890290052696295
nn.w1[1][3] = -2.6546605243784223
nn.w1[0][4] = -0.28753652311419664
nn.w1[1][4] = -2.702357339941299
nn.w1[0][5] = -0.28900543240399906
nn.w1[1][5] = -2.6553950039515217
nn.w1[0][6] = 0.8362811796297954
nn.w1[1][6] = 0.3240537371556912
nn.w2[0][0] = 1.6589350302508084
nn.w2[1][0] = 1.2051926692251191
nn.w2[2][0] = -2.398733510691691
nn.w2[3][0] = 1.5501671665020704
nn.w2[4][0] = 1.6256827951432355
nn.w2[5][0] = 1.5513111987646977
nn.w2[6][0] = 0.9044315171736391
# insert your values end.

print ("Start")

try:
    t4r.waitUntilTouch(ts)
    time.sleep(2)

    vehicle.setSpeed(20)
    vehicle.forward()

    while not ts.isTouched():
        time.sleep(0.02)
        curr_light = t4r.sumColor(cls.getColorRaw())
        result = nn.executeFunction([curr_light])[0]
        if (result < 0.375):
            result = STRAIGHT
        elif (result >= 0.375 and result <= 0.625):
            result = RIGHT  # 6
        else:
            result = LEFT # -6
        if (result != STRAIGHT):
            vehicle.stop()
            vehicle.rotate(result)
            while vehicle.isMoving():
                pass
        vehicle.forward()

except KeyboardInterrupt:
    pass
finally:
    vehicle.stop()
    BP.reset_all()
    
print ("End")



