#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Roboter/Interface")
from threading import Thread
import time
import os
import tkinter as tk
from idlelib.ToolTip import *
from NeuralNetwork import NeuralNetwork

########################################
# Program to train a neural network.   #
# file: LineTrackingTraining.pyw       #
# Author: Th. Kaffka, Cologne, Germany #
# Date: 06/08/2016                     #
########################################

PROG_NAME = "LineTracking Training"
PROG_VERSION = "0.1"
PIC_WIDTH = 659
PIC_HEIGHT = 494
COORD_ORIGIN_X = 50
COORD_ORIGIN_Y = 50

do_training = True
examples = []
examples_no = 0
factor = 0.0
error = []
item = []

class LineTrackingTraining(tk.Frame):

   def __init__(self, master=None):
      tk.Frame.__init__(self, master)
      self.grid()
      self.createWidgets()
      self.readExamples()
      self.nn = NeuralNetwork(1, 1, 0.3, 6, examples, examples_no, factor, 1)
      self.exampLabel.configure(text = str(examples_no))
      self.messageLabel.configure(text = "examples read.")
      self.printCoordinateSystem()
      
   def createWidgets(self):
      self.picture_ground = tk.Canvas(self, height=str(PIC_HEIGHT),\
                                      width=str(PIC_WIDTH), background="#ADD8E6")

      self.learnStepLabelText = tk.Label(self, text="Learn Step:")
      self.learnStepLabel = tk.Label(self, text="0")
      self.exampLabelText = tk.Label(self, text="Examples No.:")
      self.exampLabel = tk.Label(self, text="0")
      self.maxErrorLabelText = tk.Label(self, text="max. Error:")
      self.maxErrorLabel = tk.Label(self, text="0.0")
      self.minErrorLabelText = tk.Label(self, text="min. Error:")
      self.minErrorLabel = tk.Label(self, text="0.0")
      self.avErrorLabelText = tk.Label(self, text="av. Error:")
      self.avErrorLabel = tk.Label(self, text="0.0")

      self.startButton = tk.Button(self, text="Start", command=self.startTraining)
      ListboxToolTip(self.startButton, ["start the training", "of the examples."])
      self.stopButton = tk.Button(self, text="Stop", command=self.stopTraining)
      ListboxToolTip(self.stopButton, ["stops the training", "of the examples."])
      self.saveButton = tk.Button(self, text="Save", command=self.saveData)
      ListboxToolTip(self.saveButton, ["saves the", "neural network data."])
      self.resetButton = tk.Button(self, text="Reset", command=self.resetNN)
      ListboxToolTip(self.resetButton, ["resets the", "neural network."])
      self.quitButton = tk.Button(self, text="Quit", command=self.winClose)
      ListboxToolTip(self.quitButton, ["exits the program."])

      self.spacerLabel = tk.Label(self, text=" ")
      self.messageLabel = tk.Label(self, text="")

      self.picture_ground.grid(column=0, row=0, columnspan=7)

      self.learnStepLabelText.grid(column=0, row=1, sticky="w")
      self.learnStepLabel.grid(column=1, row=1, sticky="w")
      self.exampLabelText.grid(column=4, row=1, sticky="w")
      self.exampLabel.grid(column=5, row=1, sticky="w")
      self.maxErrorLabelText.grid(column=0, row=2, sticky="w")
      self.maxErrorLabel.grid(column=1, row=2, sticky="w")
      self.minErrorLabelText.grid(column=2, row=2, sticky="w")
      self.minErrorLabel.grid(column=3, row=2, sticky="w")
      self.avErrorLabelText.grid(column=4, row=2, sticky="w")
      self.avErrorLabel.grid(column=5, row=2, sticky="w")

      self.startButton.grid(column=0, row=3, sticky="w")
      self.stopButton.grid(column=1, row=3, sticky="w")
      self.saveButton.grid(column=2, row=3, sticky="w")
      self.resetButton.grid(column=3, row=3, sticky="w")
      self.quitButton.grid(column=6, row=3, sticky="e")
      
      self.spacerLabel.grid(column=0, row=4, sticky="w", columnspan=5)
      self.messageLabel.grid(column=0, row=5, sticky="w", columnspan=5)

   ####################
   # exits the program.
   def winClose(self):
#      self.quit()
      sys.exit("program closed.")

   ######################
   # reads the examples.
   def readExamples(self):
      f = open ("../data/Data.dat", "r")
      data = f.read()
      f.close()
      global examples, examples_no, factor
      exp_line = []
      number = ""
      exp_max = -1
      for c in data:
         if (c == "\n"):
            if (int(number) == 0):
                number = "0.25"
            elif (int(number) == 6):
                number = "0.50"
            elif  (int(number) == -6):
                number = "0.75"
#            exp_max = max(exp_max, float(number))
            exp_line.append(float(number))
            number = ""
            examples.append(exp_line)
            exp_line = []
            examples_no += 1
            continue
         if (c == ","):
            exp_max = max(exp_max, int(number))
            exp_line.append(int(number))
            number = ""
            continue
         number = number + c
      factor = 1.0 / (exp_max + 1.0)

   ############################
   # resets the neural network.
   def resetNN(self):
      self.nn = NeuralNetwork(1, 1, 0.3, 6, examples, examples_no, factor, 1)
      self.messageLabel.configure(text = "reset the neural network.")

   ######################
   # starts the training.
   def startTraining(self):
      global do_training
      do_training = True
      self.t = Thread(target=self.loopTraining, args=(self, PIC_WIDTH, PIC_HEIGHT,))
      self.t.start()
      self.messageLabel.configure(text = "start training.")

   #####################
   # stops the training.
   def stopTraining(self):
      global do_training
      do_training = False
      self.t.join(timeout=0.7)
      self.messageLabel.configure(text = "stop training.")

   ################################
   # saves the neural network data.
   def saveData(self):
      try:
         f = open ("../data/ltsource.txt", "w")
         f.write("factor = " + str(factor) + "\r\n")
         for i in range(0, self.nn.nh + 1):
            for j in range(0, self.nn.x_columns + 1):
               f.write("w1[" + str(j) + "][" + str(i) + "] = " \
                       + str(self.nn.w1[j][i]) + "\r\n")
         for i in range(0, self.nn.y_columns):
            for j in range(0, self.nn.nh + 1):
               f.write("w2[" + str(j) + "][" + str(i) + "] = " \
                       + str(self.nn.w2[j][i]) + "\r\n")
      finally:
         f.close()
      self.messageLabel.configure(text = "save source data.")

   ################################
   # saves the neural network data.
   def loopTraining(self, selfkw, PIC_WIDTH, PIC_HEIGHT):
      global error
      print_train_step = 0
      while (do_training):
         self.nn.learn()
         self.nn.train_step += 1
         print_train_step += 1
         if (print_train_step == 500):
            print_train_step = 0
            self.learnStepLabel.configure(text = str(self.nn.train_step))
            self.maxErrorLabel.configure(text = str(self.nn.max_error)[0:10])
            self.minErrorLabel.configure(text = str(self.nn.min_error)[0:10])
            self.avErrorLabel.configure(text = str(self.nn.av_error)[0:10])
            error.append(self.nn.av_error)
            self.printErrorValues()

   ###############################
   # prints the coordinate system.
   def printCoordinateSystem(self):
      self.picture_ground.create_line(COORD_ORIGIN_X, PIC_HEIGHT-COORD_ORIGIN_X, PIC_WIDTH, PIC_HEIGHT-COORD_ORIGIN_X, fill="#000000", tag="coord")
      self.picture_ground.create_text(PIC_WIDTH - 40, PIC_HEIGHT-10, text="Learn Step", fill="#00008B", tag="coord")
      self.picture_ground.create_line(COORD_ORIGIN_X, PIC_HEIGHT-COORD_ORIGIN_Y, COORD_ORIGIN_X, 0, fill="#000000", tag="coord")
      self.picture_ground.create_text(90, 10, text="av. Error ", fill="#00008B", tag="coord")

   ##########################
   # prints the error values.
   def printErrorValues(self):
      # delete all old lines.
      global item
      for i in item:
         self.picture_ground.delete(i)
      item = []

      # identify the min and max error.
      err_len = len(error)
      min_x = 0
      max_x = err_len
      if (err_len < 2):
         return
      max_err_y = 0.0
      min_err_y = 1.0
      for i in range(0, err_len):
         max_err_y = max(max_err_y, error[i])
         min_err_y = min(min_err_y, error[i])

      # calculate graphic factors.
      cdistance = PIC_WIDTH - COORD_ORIGIN_X
      x_fac_mult = cdistance / (max_x - min_x)
      x_fac_add = COORD_ORIGIN_X
      cdistance = PIC_HEIGHT - COORD_ORIGIN_Y
      y_fac_mult = cdistance / (max_err_y - min_err_y)
      y_fac_add = 0

      old_point_x = round((0 - min_x) * x_fac_mult) + x_fac_add
      old_point_y = round((error[0] - min_err_y) * y_fac_mult) + y_fac_add
      old_point_y = PIC_HEIGHT - old_point_y - COORD_ORIGIN_Y

      # print the error graph.
      for i in range(1, err_len):
         point_x = round((i - min_x) * x_fac_mult) + x_fac_add
         point_y = round((error[i] - min_err_y) * y_fac_mult) + y_fac_add
         point_y = PIC_HEIGHT - point_y - COORD_ORIGIN_Y
         item.append(self.picture_ground.create_line(old_point_x, old_point_y,\
                                         point_x, point_y, fill="#000000",\
                                         tag="error"))
         old_point_x = point_x
         old_point_y = point_y


app = LineTrackingTraining()
app.master.title(PROG_NAME + " (" + PROG_VERSION + ")")
app.mainloop()
