#!/usr/bin/python
##########################################
# program: classifying                   #
# file: Classifying.py                   #
# Author: Th. Kaffka, Cologne, Germany   #
# Date: 12.01.2021                       #
##########################################
import sys
sys.path.append("/home/pi/Ae/Roboter/Interface")
sys.path.append("/home/pi/Ae/Roboter/lib")
import brickpi3 
import time
from LegoMotor import LegoMotor
from LegoInfraRedSensor import LegoInfraRedSensor
from LegoTouchSensor import LegoTouchSensor
import tool4robot as t4r
import tool4file as t4f

BP = brickpi3.BrickPi3()

# define the hardware.
motor_belt = LegoMotor(BP.PORT_C)
motor_display = LegoMotor(BP.PORT_D)
ts = LegoTouchSensor(BP.PORT_4)
ir = LegoInfraRedSensor(BP.PORT_3)
time.sleep(10)

#define constants.
SHOW_NONE = 0
SHOW_BLACK = 90
SHOW_YELLOW = 180
SHOW_BLUE = 270
OBJ_BIG_CUBE = 1
OBJ_SMALL_CUBE = 2
OBJ_CYLINDER = 3
PATTERN_SIZE = 12
DATA_SIZE = 70

# define global vars.
example_data = ""
pattern = []
display_color = [SHOW_NONE, SHOW_BLUE, SHOW_BLACK, SHOW_YELLOW]

############################
# shows data on the display.
def display(obj):
    pos = display_color[obj]
    resetDisplay()
    motor_display.forward()
    while (motor_display.getEncoder() < pos):
        pass
    motor_display.stop()

######################
# reserts the diaplsy.
def resetDisplay():
    motor_display.backward()
    while (motor_display.getEncoder() > SHOW_NONE):
        pass
    motor_display.stop()

print ("Start")

try:
    for i in range(PATTERN_SIZE):
        pattern += [0]
    ndx = 0
    data_ndx = 0
    curr_object = OBJ_BIG_CUBE
    dist = 2

    motor_belt.stop()
    t4r.waitUntilTouch(ts)
    time.sleep(2)

    motor_belt.setSpeed(12)
    motor_belt.forward()

    motor_display.resetTachoCount()
    motor_display.setSpeed(25)
    display(curr_object)
    
    while dist > 1:
        time.sleep(0.13)
        dist = ir.getDistance()
        if (dist > 65):
            if (ndx > 0 and not ts.isTouched()):
                ndx = 100
            elif (ndx > 0 and ts.isTouched()):
                print("object: " + str(curr_object) + " | example: " + str(data_ndx))
                example_line = "("
                for i in range(PATTERN_SIZE-1):
                    example_line += str(pattern[i]) + ","
                example_line += str(pattern[PATTERN_SIZE-1]) + ")"
                if (curr_object == OBJ_BIG_CUBE):
                    example_line += ";(1,0,0)"
                elif (curr_object == OBJ_SMALL_CUBE):
                    example_line += ";(0,1,0)"
                elif (curr_object == OBJ_CYLINDER):
                    example_line += ";(0,0,1)"
                example_line += "\n"
                print("example: " + example_line)
                example_data += example_line
                data_ndx += 1
                if (data_ndx > DATA_SIZE):
                    data_ndx = 0
                    if (curr_object == OBJ_BIG_CUBE):
                        curr_object = OBJ_SMALL_CUBE
                    elif (curr_object == OBJ_SMALL_CUBE):
                        curr_object = OBJ_CYLINDER
                    elif (curr_object == OBJ_CYLINDER):
                        break
                    display(curr_object)
                for i in range(PATTERN_SIZE):
                    pattern[i] = 0
                ndx = 0
        else:
            if (ndx == 100):
                continue
            pattern[ndx] = dist
            ndx += 1
            if (ndx == PATTERN_SIZE):
                ndx = PATTERN_SIZE - 1

except KeyboardInterrupt:
    pass
finally:
    motor_belt.stop()
    resetDisplay()
    BP.reset_all()
    t4f.saveContent("../data/", "DataClassifying.dat", example_data)
    
print ("End")


