#!/usr/bin/python3
import sys
sys.path.append("/home/pi/Ae/Roboter/Interface")
sys.path.append("/home/pi/Ae/Roboter/lib")
import tkinter as tk
from idlelib.ToolTip import *
import tool4picture as t4p
from PatternRecogn import PatternRecogn

######################################################
# Program to identify the (biggest) color rectangle. #
# file: CameraIdentRect.py                           #
# Author: Th. Kaffka, Cologne, Germany               #
# Date: 26.01.2021                                   #
######################################################

PROG_NAME = "Ident Rect"
PROG_VERSION = "0.1"
PIC_WIDTH = 60 #80 #160
PIC_HEIGHT = 44 #64 #128
PIXEL_POINT = 3

class IdentRect(tk.Frame):

   def __init__(self, master=None):
      tk.Frame.__init__(self, master)
      self.grid()
      self.createWidgets()
      self.img_list = []
      self.p_recog = PatternRecogn(PIC_WIDTH, PIC_HEIGHT)

   def createWidgets(self):
      self.picture_ground = tk.Canvas(self, height=str(PIC_HEIGHT + 100), width=str(PIC_WIDTH + 140), background="#ADD8E6")

      self.identMaxButton = tk.Button(self, text="Identify max.", command=self.identMaxPicture)
      ListboxToolTip(self.identMaxButton, ["identify the", "pasteboard cards."])
      self.identButton = tk.Button(self, text="Identify all", command=self.identPicture)
      ListboxToolTip(self.identButton, ["identify the", "pasteboard cards."])

      self.quitButton = tk.Button(self, text="Quit", command=self.winClose)
      ListboxToolTip(self.quitButton, ["exits the program."])

      self.messageLabel = tk.Label(self, text="")

      self.picture_ground.grid(column=0, row=0, columnspan=7)
      self.identMaxButton.grid(column=0, row=1, sticky="w")
      self.identButton.grid(column=1, row=1, sticky="w")
      self.quitButton.grid(column=6, row=1, sticky="e")
      self.messageLabel.grid(column=0, row=5, sticky="w", columnspan=5)

   ####################
   # exits the program.
   def winClose(self):
      sys.exit("program closed.")

   ######################################################
   # identify the pasteboard cards with a max. rectangle.
   def identMaxPicture(self):
      rect = self.p_recog.getMaxRect()
      self.img_list = self.p_recog.getPicture()
      self.printPicture()
      self.picture_ground.create_rectangle(rect[0] * PIXEL_POINT, rect[1] * PIXEL_POINT, rect[2] * PIXEL_POINT, rect[3] * PIXEL_POINT, width=1, tag="pixel")

   ####################################################
   # identify the pasteboard cards with all rectangles.
   def identPicture(self):
      rect = self.p_recog.getAllRect()
      self.img_list = self.p_recog.getPicture()
      self.printPicture()
      for x in rect:
         self.picture_ground.create_rectangle(x[0] * PIXEL_POINT, x[1] * PIXEL_POINT, x[2] * PIXEL_POINT, x[3] * PIXEL_POINT, width=1, tag="pixel")

   #####################
   # prints the picture.
   def printPicture(self):
      for row in range(0, PIC_HEIGHT):
         for col in range(0, PIC_WIDTH):
            self.printPixel(row, col)

   #####################
   # prints the pixel.
   def printPixel(self, row, col):
      ndx = (row * PIC_WIDTH) + col
      pixel = self.img_list[ndx]
      row = row * PIXEL_POINT
      col = col * PIXEL_POINT
      for pix_row in range(0, PIXEL_POINT):
         for pix_col in range(0, PIXEL_POINT):
            self.picture_ground.create_rectangle(col + pix_col, row + pix_row, col+1 + pix_col, row+1 + pix_row, width=0, fill=t4p.makeColor(pixel[0], pixel[1], pixel[2]), tag="pixel")

app = IdentRect()
app.master.title(PROG_NAME + " (" + PROG_VERSION + ")")
app.mainloop()
