#!/usr/bin/python3
##################################################
# Program to initialize the raspberry pi camera. #
# file: CameraInit.pyw                           #
# Author: Th. Kaffka, Cologne, Germany           #
# Date: 21.01.2021                               #
##################################################

import sys
sys.path.append("/home/pi/Ae/Roboter/Interface")
sys.path.append("/home/pi/Ae/Roboter/lib")
import tkinter as tk
from idlelib.ToolTip import *
from Camera import Camera
import tool4picture as t4p
import tool4file as t4f

PROG_NAME = "Photo"
PROG_VERSION = "0.1"
PIC_WIDTH = 60 #80 #160
PIC_HEIGHT = 44 #64 #128
PIXEL_POINT = 3

STATUS_COMMON = 0
STATUS_INIT = 1

class Photo(tk.Frame):

   def __init__(self, master=None):
      tk.Frame.__init__(self, master)
      self.cam = Camera()
      self.grid()
      self.createWidgets()
      self.img_list = []
      self.init_list = []
      self.status = STATUS_COMMON
      self.init_area_x = 1
      self.init_area_y = (PIC_HEIGHT * PIXEL_POINT) + 15

   def createWidgets(self):
      self.picture_ground = tk.Canvas(self, height=str(PIC_HEIGHT + 200), width=str(PIC_WIDTH + 140), background="#ADD8E6")
      self.picture_ground.bind("<1>", self.OnMouseDown)

      self.initButton = tk.Button(self, text="Initialize", command=self.initPicture)
      ListboxToolTip(self.initButton, ["initializes", "the camera."])
      self.undoButton = tk.Button(self, text="Undo", command=self.undoPicture)
      ListboxToolTip(self.undoButton, ["undo the", "init step."])
      self.saveButton = tk.Button(self, text="Save", command=self.savePicture)
      ListboxToolTip(self.saveButton, ["save the", "init data."])

      self.quitButton = tk.Button(self, text="Quit", command=self.winClose)
      ListboxToolTip(self.quitButton, ["exits the program."])

      self.messageLabel = tk.Label(self, text="")

      self.picture_ground.grid(column=0, row=0, columnspan=7)
      self.initButton.grid(column=0, row=1, sticky="w")
      self.undoButton.grid(column=1, row=1, sticky="w")
      self.saveButton.grid(column=2, row=1, sticky="w")
      self.quitButton.grid(column=6, row=1, sticky="e")
      self.messageLabel.grid(column=0, row=5, sticky="w", columnspan=5)

   ####################
   # exits the program.
   def winClose(self):
      del self.cam
      sys.exit("program closed.")

   ####################################################
   # the mouse event of the canvas (x=width, y=height).
   def OnMouseDown(self, event):
      if (not self.status == STATUS_INIT):
         return

      if (self.init_area_y > (PIC_HEIGHT * PIXEL_POINT) + 85):
         return

      x = int(event.x / PIXEL_POINT)
      y = int(event.y / PIXEL_POINT)
      if (x > PIC_WIDTH-1 or y > PIC_HEIGHT-1):
         return

      # create next color init
      ndx = (y * PIC_WIDTH) + x
      pixel = self.img_list[ndx]
      if (self.init_list.count(pixel) == 0):
         self.init_list.append(pixel)
         self.picture_ground.create_rectangle(self.init_area_x, self.init_area_y, self.init_area_x + 15, self.init_area_y + 15, width=1, fill=t4p.makeColor(pixel[0], pixel[1], pixel[2]), tag="pixel")
         self.init_area_x += 20
         if (self.init_area_x == 161):
            self.init_area_x = 1
            self.init_area_y += 20

      # color the graphic.
      length = len(self.img_list)
      for x in range(0, length):
         if (self.img_list[x] == pixel):
            self.img_list[x] = (255, 255, 255)
      self.printPicture()

   #####################
   # save the init data.
   def savePicture(self):
      t4f.saveContent("../data/", "initcamera.txt", str(self.init_list))

   #########################
   # initializes the camera.
   def initPicture(self):
      self.status = STATUS_INIT
      self.img_list = self.cam.captureList(PIC_WIDTH, PIC_HEIGHT)
      self.init_list = []
      self.init_area_x = 1
      self.init_area_y = (PIC_HEIGHT * PIXEL_POINT) + 15
      self.printPicture()

   #####################
   # prints the pixel.
   def printPixel(self, row, col):
      ndx = (row * PIC_WIDTH) + col
      pixel = self.img_list[ndx]
      row = row * PIXEL_POINT
      col = col * PIXEL_POINT
      for pix_row in range(0, PIXEL_POINT):
         for pix_col in range(0, PIXEL_POINT):
            self.picture_ground.create_rectangle(col + pix_col, row + pix_row, col+1 + pix_col, row+1 + pix_row, width=0, fill=t4p.makeColor(pixel[0], pixel[1], pixel[2]), tag="pixel")

   #####################
   # prints the picture.
   def printPicture(self):
      for row in range(0, PIC_HEIGHT):
         for col in range(0, PIC_WIDTH):
            self.printPixel(row, col)

   ###############################
   # undo the initialization step.
   def undoPicture(self):
      if (self.init_area_y == PIC_HEIGHT + 15 and self.init_area_x == 1):
        self.init_area_x = 1
        self.init_area_y = PIC_HEIGHT + 15
        return
      ndx = len(self.init_list) - 1
      del self.init_list[ndx]
      if (self.init_area_x == 1):
         self.init_area_x = 161
         self.init_area_y -= 20
      self.init_area_x -= 20
      self.picture_ground.create_rectangle(self.init_area_x, self.init_area_y, self.init_area_x + 16, self.init_area_y + 16, width=0, fill="#ADD8E6", tag="pixel")

app = Photo()
app.master.title(PROG_NAME + " (" + PROG_VERSION + ")")
app.mainloop()
