#!/usr/bin/python
########################################
# program can collector                #
# file: CanCollector.py                #
# Author: Th. Kaffka, Cologne, Germany #
# Date: 02/02/2021                     #
########################################
import sys
sys.path.append("/home/pi/Ae/Roboter/Interface")
sys.path.append("/home/pi/Ae/Roboter/lib")
import brickpi3 
import time
import tool4robot as t4r

# import classes here.
from PatternRecogn import PatternRecogn
from Led import Led
from LegoMotor import LegoMotor
from LegoTouchSensor import LegoTouchSensor
from LegoColorSensor import LegoColorSensor
from HitechnicCompassSensor import HitechnicCompassSensor

BP = brickpi3.BrickPi3()

# global constants.
PIC_WIDTH = 60       # width of camera picture.
PIC_HEIGHT = 44      # height of camera picture.
COLOR_BLACK = 1      # code of the color black.
COLOR_WHITE = 6      # code of the color white.
DIR_LEFT = 0         # code of left direction.
DIR_RIGHT = 1        # code of right direction.
FOUND_CAN = 0        # code of 'can found'.
FOUND_STOP = 1       # code of 'end status found'.
FOUND_NONE = 2       # code of 'found nothing'.
LED_IDENT = 1        # signal identify.
LED_TRANS = 2        # signal tranesfer.
LED_END = 3          # signal end.


# parameter.
MOTOR_SPEED = 20     # the speed of the vehicle motors.
MOTOR_SPEED_SLOW = MOTOR_SPEED - 5 # the slow speed of the vehicle motors.
CLAW_SPEED = 50      # the speed of the claw motor.
GRIP_WIDTH = 90      # the width of the grip of the claw.
DIR_CORR = 7         # correction of the direction of the compass.
MID_CORR = 8         # correction of the picture mid.
CAN_DISTANCE = 29    # the distance of a can.

# global valibales.
compass_straight = 0
compass_left = 0
compass_right = 0
compass_back = 0
led_signal = 0

# define the hardware.
p_recog = PatternRecogn(PIC_WIDTH, PIC_HEIGHT)
led = Led()
go = LegoTouchSensor(BP.PORT_2)
color = LegoColorSensor(BP.PORT_4)
compass = HitechnicCompassSensor(BP.PORT_1)
motor_left = LegoMotor(BP.PORT_C)
motor_right = LegoMotor(BP.PORT_B)
motor_claw = LegoMotor(BP.PORT_D)
time.sleep(10)

#############################
# led signal: can identified.
def signalIdentify():
   global led_signal
   if (led_signal == LED_IDENT):
      return
   led_signal = LED_IDENT
   led.blinkOff()
   time.sleep(0.5)
   led.blinkOn(0.1, 0.1);
   
###########################################
# led signal: transfer to collecting point.
def signalTransfer():
   global led_signal
   if (led_signal == LED_TRANS):
      return
   led_signal = LED_TRANS
   led.blinkOff()
   time.sleep(0.5)
   led.blinkOn(0.1, 0.3);

#############################
# led signal: end collecting.
def signalEnd():
   global led_signal
   if (led_signal == LED_END):
      return
   led_signal = LED_END
   led.blinkOff()
   time.sleep(0.5)
   led.blinkOn(0.4, 0.3);

##############################################
# initalize the main directions of the compass.
def initDirection():
   global compass_straight, compass_left, compass_right, compass_back
   time.sleep(0.02)
   compass_straight = compass.getDirection()
   compass_left = corrDirection(compass_straight + 270)
   compass_right = corrDirection(compass_straight + 90)
   compass_back = corrDirection(compass_straight + 180)

###################################
# correcting the direction setting.
def corrDirection(direction):
   if (direction > 359):
      direction -= 360
   elif (direction < 0):
      direction += 360
   return direction

###############################################
# move forward, since a white bar ist detected.
def forwardSinceBar():
   motor_left.backward()
   motor_right.backward()
   while (not go.isTouched()):
      compass.getDirection()
      light = color.getColor()
      if (light == COLOR_WHITE):
         break
   motor_left.stop()
   motor_right.stop()

################################################
# turn the vehicle, since target_dir is reached.
def turnSinceDir(direction, target_dir):
   if (direction == DIR_LEFT):
      motor_left.forward()
      motor_right.backward()
   elif (direction == DIR_RIGHT):
      motor_left.backward()
      motor_right.forward()
   while (not go.isTouched()):
      curr_dir = compass.getDirection()
      if (curr_dir - DIR_CORR <= target_dir and curr_dir + DIR_CORR >= target_dir):
         break
   motor_left.stop()
   motor_right.stop()

###########################
# grip a can with the claw.
def clawGrip():
   pos = motor_claw.getEncoder() 
   target = pos + GRIP_WIDTH
   motor_claw.forward()
   while (pos < target):
      pos = motor_claw.getEncoder()
      compass.getDirection()
   motor_claw.stop()

##############################
# loose the can with the claw.
def clawUnGrip():
   pos = motor_claw.getEncoder() 
   target = pos - GRIP_WIDTH
   motor_claw.backward()
   while (pos > target):
      pos = motor_claw.getEncoder()
      compass.getDirection()
   motor_claw.stop()

######################
# get the lowest rect.
def getLowRect():
   rect_list = p_recog.getAllRect()
   if (len(rect_list) > 0):
      ndx = lowRect(rect_list)
      return rect_list[ndx]
   else:
      return (0, 0, 0, 0)

################################
# identify the lowest rectangle.
def lowRect(rect_list):
   ndx = 0
   low_r = rect_list[0]
   length = len(rect_list)
   for x in range(1, length):
      low_next = rect_list[x]
      if (low_next[3] < low_r[3]):
         ndx = x
         low_r = low_next
   return ndx

############
# move back.
def moveBack(sec):
   motor_left.forward()
   motor_right.forward()
   end = time.time() + sec
   while (end > time.time()):
      compass.getDirection()
   motor_left.stop()
   motor_right.stop()

##########################################
# scanns the environment for cans or stop.
def scannCan():
   dir_left = corrDirection(compass_straight - 45)
   dir_right = corrDirection(compass_straight + 15)
   
   while (not go.isTouched()):
      turnSinceDir(DIR_LEFT, dir_left)
      time.sleep(1.0)
      # turn right since can found or dir_right.
      motor_left.backward()
      motor_right.forward()
      while (not go.isTouched()):
         curr_dir = compass.getDirection()
         if (curr_dir - DIR_CORR <= dir_right and curr_dir + DIR_CORR >= dir_right):
            break
         rect = p_recog.getMaxRect()
         if (not rect == (0, 0, 0, 0)):
            motor_left.stop()
            motor_right.stop()
            time.sleep(1.0)
            return FOUND_CAN
      motor_left.stop()
      motor_right.stop()
      time.sleep(1.0)

      # move a bit forward.
      turnSinceDir(DIR_LEFT, compass_straight)
      motor_left.backward()
      motor_right.backward()
      end = time.time() + 1.5
      while (end > time.time()):
         light = color.getColor()
         compass.getDirection()
         if (light == COLOR_WHITE):
            motor_left.stop()
            motor_right.stop()
            time.sleep(1.0)
            return FOUND_STOP
      motor_left.stop()
      motor_right.stop()
      time.sleep(1.0)
   return FOUND_NONE

##################
# move to the can.
def goToCan():
   # center the can.
   motor_left.setSpeed(MOTOR_SPEED_SLOW)
   motor_right.setSpeed(MOTOR_SPEED_SLOW)
   rect = p_recog.getMaxRect()
   while (rect == (0, 0, 0, 0)):
      print("go to can: not found 1")
      motor_left.backward()
      motor_right.backward()
      compass.getDirection()
      time.sleep(0.3)
      motor_left.stop()
      motor_right.stop()
      rect = p_recog.getMaxRect()
   mid_rect = rect[0] + (rect[2] - rect[0]) / 2
   if (mid_rect < mid_pic):
      compass.getDirection()
      motor_left.forward()
      motor_right.backward()
   elif (mid_rect > mid_pic):
      compass.getDirection()
      motor_left.backward()
      motor_right.forward()
   while (not (mid_pic - MID_CORR < mid_rect and mid_pic + MID_CORR > mid_rect)):
      compass.getDirection()
      rect = p_recog.getMaxRect()
      if (rect == (0, 0, 0, 0)):
         print("go to can: not found 2")
         continue
      mid_rect = rect[0] + (rect[2] - rect[0]) / 2
   motor_left.stop()
   motor_right.stop()
   time.sleep(1.0)

   motor_left.setSpeed(MOTOR_SPEED)
   motor_right.setSpeed(MOTOR_SPEED)

   # go to the can.
   motor_left.backward()
   motor_right.backward()
   while (True):
      time.sleep(0.04)
      compass.getDirection()
      rect = getLowRect()
      if (rect[3] > CAN_DISTANCE):
         break
   motor_left.stop()
   motor_right.stop()
   time.sleep(1.0)

   # grap the can.
   motor_left.backward()
   motor_right.backward()
   end = time.time() + 2.5
   while (end > time.time()):
      compass.getDirection()
      time.sleep(0.04)
   motor_left.stop()
   motor_right.stop()
   time.sleep(1.0)

###########################
# move the can to the sink.
def moveToSink():
   clawGrip()
   time.sleep(1.0)

   turnSinceDir(DIR_RIGHT, compass_right)
   time.sleep(1.0)
   forwardSinceBar()
   time.sleep(1.0)

   turnSinceDir(DIR_RIGHT, compass_back)
   time.sleep(1.0)
   forwardSinceBar()
   clawUnGrip()
   time.sleep(1.0)
   
   moveBack(2.5)
   time.sleep(1.0)
   turnSinceDir(DIR_RIGHT, compass_straight)
   time.sleep(1.0)

print ("Start")

try:

   initDirection()
   motor_left.setSpeed(MOTOR_SPEED)
   motor_right.setSpeed(MOTOR_SPEED)
   motor_claw.setSpeed(CLAW_SPEED)
   mid_pic = PIC_WIDTH / 2 

   # start a collecting session.
   signalIdentify()
   t4r.waitUntilTouch(go)
   time.sleep(2)

   # test the claw.
   clawGrip()
   time.sleep(1.0)
   clawUnGrip()
   time.sleep(1.0)

   while (not go.isTouched()):
      result = scannCan()
      if (result == FOUND_CAN):
         print ("can found")
         signalTransfer()
         goToCan()
         moveToSink()
      elif (result == FOUND_STOP):
         signalEnd()
         break

except KeyboardInterrupt:
   pass
finally:
   led.blinkOff()
   motor_left.stop()
   motor_right.stop()
   motor_claw.stop()
   BP.reset_all()

print ("End")
